#ifndef vxa_kernel_hist_intersect_h
#define vxa_kernel_hist_intersect_h

namespace vxa
{
  class KernelHistIntersect2 : public KernelFunction
  {
  public:
    Array<double> w; // parameters
    // KernelFunction.parameter holds the representation of these parameters


    KernelHistIntersect2* CloneI() 
    {
      KernelHistIntersect2* pk = new KernelHistIntersect2(w.nelem());
      pk->w.resize(w.nelem(),false);
      // vxa: add a comment explaining why copying parameters is not necessary
      /*
      for(size_t i=0; i < parameter.nelem(); ++i)
      {
        pk->setParameter(i,getParameter(i));
      }
      */
      return pk;
    }

    KernelHistIntersect2(int dim)
    {
      parameter.resize(dim, false);
      parameter = 1; 
      this->w.resize(dim, false);
      this->w = 1; 
    }

    ~KernelHistIntersect2()
    {
    }
    
    double eval(const Array<double>& x1, const Array<double>& x2) const
    {
      unsigned int N = parameter.dim(0);


      double k=0;
      for(unsigned int n=0; n < N; ++n)
      {
        k += w(n)*Shark::min(x1(n),x2(n));
      }

      return k;
    }

    double evalDerivative(const Array<double>& x1, 
      const Array<double>& x2, Array<double>& derivative) const
    {
      unsigned int N = parameter.dim(0);
      derivative.resize(N, false);
      double k=0.0;

      for(unsigned int n=0; n < N; ++n)
      {
        double x = Shark::min(x1(n),x2(n));
        k += w(n)*x;
        derivative(n) = x * parameter(n); 
      }

      return k;
    }

    void setParameter(unsigned int index, double value)
    {
      parameter(index) = value;
      w(index) = value * value;
    }


  }; // KernelHistIntersect2

  struct KernelHistIntersect : public KernelFunction
  {

    KernelHistIntersect(int dim)
    {
      parameter.resize(dim, false);
      parameter = 0; // log-scale
      this->w.resize(dim, false);
      this->w = 1; // exp(parameter)
    }

    ~KernelHistIntersect()
    {
    }

    double eval(const Array<double>& x1, const Array<double>& x2) const
    {
      unsigned int N = parameter.dim(0);


      double k=0;
      for(unsigned int n=0; n < N; ++n)
      {
        k += w(n)*Shark::min(x1(n),x2(n));
      }

      return k;
    }


    double evalDerivative(const Array<double>& x1, 
      const Array<double>& x2, Array<double>& derivative) const
    {
      unsigned int N = parameter.dim(0);
      derivative.resize(N, false);
      double k=0.0;

      for(unsigned int n=0; n < N; ++n)
      {
        double x = Shark::min(x1(n),x2(n));
        k += w(n)*x;
        derivative(n) = x; // dk/log(w)
      }

      return k;
    }


    void setParameter(unsigned int index, double value)
    {
      parameter(index) = value;
      w(index) = ::exp(value);
    }

    Array<double> w;
  };


  // ************************************************


  // kernel hist intersect with all weights equal to one
  class KernelHistIntersect_nw : public KernelFunction 
  {
  public:
    KernelHistIntersect_nw(void)
    {
      parameter.resize(0, false);
    }

    ~KernelHistIntersect_nw()
    {
    }

    KernelHistIntersect_nw* CloneI() 
    {
      KernelHistIntersect_nw* pk = new KernelHistIntersect_nw();
      return pk;
    }


    double eval(const Array<double>& x1, const Array<double>& x2) const
    {
      unsigned int N = x1.dim(0);
      double k=0;
      for(unsigned int n=0; n < N; ++n)
      {
        k += Shark::min(x1(n),x2(n));
      }

      return k;
    }
    double evalDerivative(const Array<double>& x1, 
      const Array<double>& x2, Array<double>& derivative) const
    {
      derivative.resize(0, false);
      return eval(x1,x2);
    }

    void setParameter(unsigned int index, double value)
    {
      throw SHARKEXCEPTION("kernel has no parameters!");
    }

  };


  //
  // **************************************
  //

  struct KernelHistIntersect_feat_subset : public KernelFunction 
  {
    // subset of indices of features used by this kernel
    unsigned int ndx_begin;
    unsigned int ndx_end;

    KernelHistIntersect_feat_subset(unsigned int begin, unsigned int end)
      : ndx_begin(begin), ndx_end(end)
    {
      parameter.resize(0, false);
    }

    ~KernelHistIntersect_feat_subset()
    {
    }


    double eval(const Array<double>& x1, const Array<double>& x2) const
    {
      double k = 0.0;

      for(unsigned int n=ndx_begin; n < ndx_end; ++n)
      {
        k += Shark::min(x1(n),x2(n));
      }

      return k;
    }

    double evalDerivative(const Array<double>& x1, 
      const Array<double>& x2, Array<double>& derivative) const
    {
      //throw SHARKEXCEPTION("kernel has no parameters!");
      derivative.resize(0, false);
      return eval(x1,x2);
    }

    void setParameter(unsigned int index, double value)
    {
      throw SHARKEXCEPTION("kernel has no parameters!");
    }


  };

}

#endif //#ifndef vxa_kernel_hist_intersect_h