#ifndef vxa_kernel_simplex_h
#define vxa_kernel_simplex_h

#include <SharkDefs.h>
#include <BasicMath.h>
#include <Array/Array.h>
#include <ReClaM/KernelFunction.h>

namespace vxa
{
  double simplex_dist(const Array<double>& x1, const Array<double>& x2);

  double simplex_dist(const Array<double>& x1, const Array<double>& x2,
    const Array<double>& w /*, Array<double>& tmp1, Array<double>& tmp2 */);

  double kernel_simplex_est_bandwidth(const Array<double>& x);

  void test_kernel_simplex_fix_gamma(void);

  // only weights change; bandwidth is fixed
  class KernelSimplex_fix_gamma : public KernelFunction
  {
  public:
    Array<double> w;

    // KernelFunction.parameter holds the representation

    double g; // gamma
    void setGamma(double gamma){g=gamma;}
    double getGamma(void) {return g;}

    KernelSimplex_fix_gamma* CloneI() 
    {
      KernelSimplex_fix_gamma* pk = new KernelSimplex_fix_gamma(w.nelem(),g);
      return pk;
    }


    KernelSimplex_fix_gamma(int n_dim, double gamma)
    {
      setGamma(gamma);
      parameter.resize(n_dim, false);
      parameter = 1; 
      this->w.resize(n_dim, false);
      this->w = 1; 
    }

    void setParameter(unsigned int index, double value);

    ~KernelSimplex_fix_gamma()
    {
    }

    double eval(const Array<double>& x1, const Array<double>& x2) const;

    double evalDerivative(const Array<double>& x1, 
      const Array<double>& x2, Array<double>& derivative) const;


  };


  // inefficient version: do not use 
 // learn \gamma and \lambda
  class KernelSimplex : public KernelFunction
  {
  public:
    // this->parameter is: \gamma, \lambda
    Array<double> w; // only for lambda
    double g;

    void setGamma(double gamma){g=gamma;}
    double getGamma(void) {return g;}

    KernelSimplex* CloneI() 
    {
      KernelSimplex* pk = new KernelSimplex(w.nelem(),::exp(w(0)));
      //pk->w.resize(w.nelem(),false); // <- done in the constructor
      // copy happens elsewhere
      return pk;
    }


    KernelSimplex(int n_dim, double gamma)
    {
      parameter.resize(n_dim+1, false); // \gamma, \lambda
      parameter = 0; // log-space
      parameter(0) = ::log(gamma); 
      this->g = gamma;
      this->w.resize(n_dim, false);
      this->w = 1; 
    }

    void setParameter(unsigned int index, double value);

    ~KernelSimplex()
    {
    }

    double eval(const Array<double>& x1, const Array<double>& x2) const;

    double evalDerivative(const Array<double>& x1, 
      const Array<double>& x2, Array<double>& derivative) const;


  };




  // old code. do NOT use
  // only one parameter: bandwidth
  struct KernelSimplex_fix_lambda : public KernelFunction
  {
    Array<double> w;

    KernelSimplex_fix_lambda(void)
    {
      parameter.resize(1, false);
      parameter = 1; 
      this->w.resize(1, false);
      this->w = 1; 
    }

    ~KernelSimplex_fix_lambda()
    {
    }

    void setParameter(unsigned int index, double value)
    {
      parameter(index) = value;
      w(index) = value * value;
      //w(index) = ::exp(value);
    }

    double eval(const Array<double>& x1, const Array<double>& x2) const
    {
      unsigned int N = x1.nelem();

      double d=0;
      for(unsigned int n=0; n < N; ++n)
      {
        d += ::sqrt(x1(n)*x2(n));
      }
      d = Shark::min(1.0,d); // must be in [0,1]
      d = ::acos(d);

      double gamma = w(0);
      double k = exp(-0.5*vxa::Core::sq(d/gamma));
      return k;
    }

    double evalDerivative(const Array<double>& x1, 
      const Array<double>& x2, Array<double>& derivative) const
    {
      unsigned int N = x1.nelem();
      derivative.resize(1, false);

      double d = simplex_dist(x1,x2);
      double d2 = d*d;
      double gamma = w(0);
      double g2 = gamma*gamma;
      double k = exp(-0.5*vxa::Core::sq(d/gamma));

      derivative(0) = k * d2 * 1.0/g2 * 1.0/parameter(0);

      return k;
    }


  };
}

#endif // vxa_kernel_simplex_h

